#ifndef ONEWIRE_H
#define	ONEWIRE_H
//------------------------------------------------------------------------------
#include "oneWire_pins.h"
//------------------------------------------------------------------------------
void oneWire_init(void)
{
    T1CON  = 0x10;        // set Timer1 clock source to internal with 1:2 prescaler (Timer1 clock = 1MHz)
     
    oneWire_timer1Reset();
}
//------------------------------------------------------------------------------
void oneWire_startSignal(void) 
{
  DATA_PIN_DIR = 0;     // configure DHT11_PIN as output
  DATA_PIN = 0;         // clear DHT11_PIN output (logic 0)

  __delay_ms(25);        // wait 25 ms
  DATA_PIN = 1;         // set DHT11_PIN output (logic 1)

  __delay_us(30);        // wait 30 us
  DATA_PIN_DIR = 1;     // configure DHT11_PIN as input
}
//------------------------------------------------------------------------------
void oneWire_timer1Reset()
{
  TMR1H = 0;             // reset Timer1
  TMR1L = 0;
}
//------------------------------------------------------------------------------
__bit oneWire_checkResponse() 
{
  __bit static response = 0;
  
  oneWire_timer1Reset();
  TMR1ON = 1;                        // enable Timer1 module

  while(!DATA_PIN && TMR1L < 100);  // wait until DHT11_PIN becomes high (checking of 80s low time response)

  if(TMR1L > 99)                     // if response time > 99S: Response error
  {
    response = 0;                    // return 0 (Device has a problem with response)
  }
  else
     {
        oneWire_timer1Reset();
        while(DATA_PIN && TMR1L < 100); // wait until DHT11_PIN becomes low (checking of 80S high time response)
        
        // if response time > 99S: Response error
        response = (TMR1L > 99) ? 0 : 1; // return 0 (Device has a problem with response). return 1 (response OK)
     }
  
  return response;
}
//------------------------------------------------------------------------------
__bit oneWire_checkTimeout1()
{
    __bit static response = 0;
    
    oneWire_timer1Reset();

    while(!DATA_PIN)      // wait until DHT11_PIN becomes high
    {
      if(TMR1L > 100)      // if low time > 100:  Time out error (Normally it takes 50s)
      {    
        response = 1; break;
      }
    }
    
    return response;
}
//------------------------------------------------------------------------------
__bit oneWire_checkTimeout2()
{
    __bit static response = 0;
    
    oneWire_timer1Reset();

    while(DATA_PIN)       // wait until DHT11_PIN becomes low
    {
      if(TMR1L > 100)      // if high time > 100:  Time out error (Normally it takes 26-28s for 0 and 70s for 1)
      {    
        response = 1; break;       // return 1 (timeout error)
      }
    } 
    
    return response;
}
//------------------------------------------------------------------------------
    __bit oneWire_readByte(unsigned char* data)
{
  __bit static response = 0;
  *data = 0;
  
  for(char i = 0; i < 8; i++)
  {      
    if(oneWire_checkTimeout1() || oneWire_checkTimeout2())
    {
        response = 1; break;            // timeout error
    }

    if(TMR1L > 50)                      // if high time > 50:  Sensor sent 1
    { 
       *data += (0x01 << (7 - i));    // set bit (7 - i)
    }
  }

  return response;                      // if returned 0 (data read OK)
}
//------------------------------------------------------------------------------
#endif	/* ONEWIRE_H */